@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2003-2007 Imendio AB
+ * Copyright (C) 2007 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +22,7 @@
  *          Richard Hult <richard@imendio.com>
  *          Martyn Russell <martyn@imendio.com>
  *          Geert-Jan Van den Bogaerde <geertjan@gnome.org>
+ *          Xavier Claessens <xclaesse@gmail.com>
  */
 
 #include "config.h"
@@ -32,21 +34,24 @@
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
+#include <libempathy/empathy-contact-manager.h>
+#include <libempathy/empathy-contact-list.h>
 #include <libempathy/gossip-contact.h>
 #include <libempathy/gossip-debug.h>
 #include <libempathy/gossip-message.h>
 #include <libempathy/gossip-conf.h>
 
 #include "gossip-chat-window.h"
+#include "empathy-images.h"
 //#include "gossip-add-contact-dialog.h"
 //#include "gossip-chat-invite.h"
 //#include "gossip-contact-info-dialog.h"
 //#include "gossip-log-window.h"
-//#include "gossip-new-chatroom-dialog.h"
+#include "gossip-new-chatroom-dialog.h"
 #include "gossip-preferences.h"
 #include "gossip-private-chat.h"
+#include "gossip-group-chat.h"
 //#include "gossip-sound.h"
-#include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_WINDOW, GossipChatWindowPriv))
@@ -104,8 +109,6 @@ struct _GossipChatWindowPriv {
 static void       gossip_chat_window_class_init         (GossipChatWindowClass *klass);
 static void       gossip_chat_window_init               (GossipChatWindow      *window);
 static void       gossip_chat_window_finalize           (GObject               *object);
-static GdkPixbuf *chat_window_get_status_pixbuf         (GossipChatWindow      *window,
-							 GossipChat            *chat);
 static void       chat_window_accel_cb                  (GtkAccelGroup         *accelgroup,
 							 GObject               *object,
 							 guint                  key,
@@ -177,6 +180,7 @@ static void       chat_window_composing_cb              (GossipChat            *
 							 GossipChatWindow      *window);
 static void       chat_window_new_message_cb            (GossipChat            *chat,
 							 GossipMessage         *message,
+							 gboolean               is_backlog,
 							 GossipChatWindow      *window);
 static GtkNotebook* chat_window_detach_hook             (GtkNotebook           *source,
 							 GtkWidget             *page,
@@ -227,7 +231,7 @@ typedef enum {
 } DndDragType;
 
 static const GtkTargetEntry drag_types_dest[] = {
-	{ "text/contact-id", GTK_TARGET_SAME_APP, DND_DRAG_TYPE_CONTACT_ID },
+	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
 	{ "GTK_NOTEBOOK_TAB", GTK_TARGET_SAME_APP, DND_DRAG_TYPE_TAB },
 };
 
@@ -273,7 +277,7 @@ gossip_chat_window_init (GossipChatWindow *window)
 	priv->tooltips = g_object_ref (gtk_tooltips_new ());
 	gtk_object_sink (GTK_OBJECT (priv->tooltips));
 
-	glade = gossip_glade_get_file ("empathy-chat.glade",
+	glade = gossip_glade_get_file ("gossip-chat.glade",
 				       "chat_window",
 				       NULL,
 				       "chat_window", &priv->dialog,
@@ -352,9 +356,9 @@ gossip_chat_window_init (GossipChatWindow *window)
 	 * stock image
 	 */
 	image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (priv->menu_conv_info));
-	gtk_image_set_from_stock (GTK_IMAGE (image),
-				  GOSSIP_STOCK_CONTACT_INFORMATION,
-				  GTK_ICON_SIZE_MENU);
+	gtk_image_set_from_icon_name (GTK_IMAGE (image),
+				      EMPATHY_IMAGE_CONTACT_INFORMATION,
+				      GTK_ICON_SIZE_MENU);
 
 	/* Set up smiley menu */
 	menu = gossip_chat_view_get_smiley_menu (
@@ -500,30 +504,6 @@ gossip_chat_window_finalize (GObject *object)
 	G_OBJECT_CLASS (gossip_chat_window_parent_class)->finalize (object);
 }
 
-static GdkPixbuf *
-chat_window_get_status_pixbuf (GossipChatWindow *window,
-			       GossipChat       *chat)
-{
-	GossipChatWindowPriv *priv;
-	GdkPixbuf            *pixbuf;
-
-	priv = GET_PRIV (window);
-
-	if (g_list_find (priv->chats_new_msg, chat)) {
-		pixbuf = gossip_stock_render (GOSSIP_STOCK_MESSAGE,
-					      GTK_ICON_SIZE_MENU);
-	}
-	else if (g_list_find (priv->chats_composing, chat)) {
-		pixbuf = gossip_stock_render (GOSSIP_STOCK_TYPING,
-					      GTK_ICON_SIZE_MENU);
-	}
-	else {
-		pixbuf = gossip_chat_get_status_pixbuf (chat);
-	}
-
-	return pixbuf;
-}
-
 static void
 chat_window_accel_cb (GtkAccelGroup    *accelgroup,
 		      GObject          *object,
@@ -666,15 +646,24 @@ static void
 chat_window_update_status (GossipChatWindow *window,
 			   GossipChat       *chat)
 {
-	GtkImage  *image;
-	GdkPixbuf *pixbuf;
+	GossipChatWindowPriv *priv;
+	GtkImage             *image;
+	const gchar          *icon_name = NULL;
 
-	pixbuf = chat_window_get_status_pixbuf (window, chat);
+	priv = GET_PRIV (window);
+
+	if (g_list_find (priv->chats_new_msg, chat)) {
+		icon_name = EMPATHY_IMAGE_MESSAGE;
+	}
+	else if (g_list_find (priv->chats_composing, chat)) {
+		icon_name = EMPATHY_IMAGE_TYPING;
+	}
+	else {
+		icon_name = gossip_chat_get_status_icon_name (chat);
+	}
 	image = g_object_get_data (G_OBJECT (chat), "chat-window-tab-image");
-	gtk_image_set_from_pixbuf (image, pixbuf);
+	gtk_image_set_from_icon_name (image, icon_name, GTK_ICON_SIZE_MENU);
 
-	g_object_unref (pixbuf);
-	
 	chat_window_update_title (window, chat);
 	chat_window_update_tooltip (window, chat);
 }
@@ -684,7 +673,6 @@ chat_window_update_title (GossipChatWindow *window,
 			  GossipChat       *chat)
 {
 	GossipChatWindowPriv	*priv;
-	GdkPixbuf		*pixbuf = NULL;
 	const gchar             *str;
 	gchar			*title;
 	gint  			 n_chats;
@@ -740,16 +728,10 @@ chat_window_update_title (GossipChatWindow *window,
 	g_free (title);
 
 	if (priv->chats_new_msg) {
-		pixbuf = gossip_stock_render (GOSSIP_STOCK_MESSAGE,
-					      GTK_ICON_SIZE_MENU);
+		gtk_window_set_icon_name (GTK_WINDOW (priv->dialog),
+					  EMPATHY_IMAGE_MESSAGE);
 	} else {
-		pixbuf = NULL;
-	}
-
-	gtk_window_set_icon (GTK_WINDOW (priv->dialog), pixbuf);
-
-	if (pixbuf) {
-		g_object_unref (pixbuf);
+		gtk_window_set_icon_name (GTK_WINDOW (priv->dialog), NULL);
 	}
 }
 
@@ -780,16 +762,11 @@ chat_window_update_menu (GossipChatWindow *window)
 	is_connected = gossip_chat_is_connected (priv->current_chat);
 
 	if (gossip_chat_is_group_chat (priv->current_chat)) {
-#if 0
-FIXME:
-		GossipGroupChat       *group_chat;
-		GossipChatroomManager *manager;
-		GossipChatroom        *chatroom;
-		GossipChatroomId       id;
-		gboolean               saved;
+		GossipGroupChat *group_chat;
+		gboolean         saved = TRUE;
+		gboolean         show_contacts;
 
 		group_chat = GOSSIP_GROUP_CHAT (priv->current_chat);
-		chatroom = gossip_group_chat_get_chatroom (group_chat);
 
 		/* Show / Hide widgets */
 		gtk_widget_show (priv->menu_room);
@@ -801,9 +778,11 @@ FIXME:
 		/* Can we add this room to our favourites and are we
 		 * connected to the room?
 		 */
+		/* FIXME:
 		manager = gossip_app_get_chatroom_manager ();
 		id = gossip_chatroom_get_id (chatroom);
 		saved = gossip_chatroom_manager_find (manager, id) != NULL;
+		*/
 
 		gtk_widget_set_sensitive (priv->menu_room_add, !saved);
 		gtk_widget_set_sensitive (priv->menu_conv_insert_smiley, is_connected);
@@ -816,7 +795,7 @@ FIXME:
 		 * with 2 or more group chat windows where showing
 		 * contacts doesn't do anything.
 		 */
-		show_contacts = gossip_chat_get_show_contacts (priv->current_chat);
+		show_contacts = gossip_group_chat_get_show_contacts (group_chat);
 
 		g_signal_handlers_block_by_func (priv->menu_room_show_contacts,
 						 chat_window_show_contacts_toggled_cb,
@@ -829,15 +808,17 @@ FIXME:
 		g_signal_handlers_unblock_by_func (priv->menu_room_show_contacts,
 						   chat_window_show_contacts_toggled_cb,
 						   window);
-#endif
 	} else {
+		GossipPrivateChat  *chat;
 		GossipSubscription  subscription;
 		GossipContact      *contact;
 
+		chat = GOSSIP_PRIVATE_CHAT (priv->current_chat);
+
 		/* Show / Hide widgets */
 		gtk_widget_hide (priv->menu_room);
 
-		contact = gossip_chat_get_contact (priv->current_chat);
+		contact = gossip_private_chat_get_contact (chat);
 		subscription = gossip_contact_get_subscription (contact);
 		if (!(subscription & GOSSIP_SUBSCRIPTION_FROM)) {
 			gtk_widget_show (priv->menu_conv_add_contact);
@@ -893,11 +874,11 @@ chat_window_add_contact_activate_cb (GtkWidget        *menuitem,
 				     GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
-	GossipContact        *contact;
+	//GossipContact        *contact;
 
 	priv = GET_PRIV (window);
 
-	contact = gossip_chat_get_contact (priv->current_chat);
+	//contact = gossip_chat_get_contact (priv->current_chat);
 
 	// FIXME: gossip_add_contact_dialog_show (NULL, contact);
 }
@@ -932,13 +913,13 @@ chat_window_info_activate_cb (GtkWidget        *menuitem,
 			      GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
-	GossipContact        *contact;
+	//GossipContact        *contact;
 
 	priv = GET_PRIV (window);
-
+/*FIXME:
 	contact = gossip_chat_get_contact (priv->current_chat);
 
-/*FIXME:	gossip_contact_info_dialog_show (contact,
+	gossip_contact_info_dialog_show (contact,
 					 GTK_WINDOW (priv->dialog));*/
 }
 
@@ -1028,7 +1009,7 @@ chat_window_show_contacts_toggled_cb (GtkWidget        *menuitem,
 	g_return_if_fail (priv->current_chat != NULL);
 
 	show = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (priv->menu_room_show_contacts));
-	gossip_chat_set_show_contacts (priv->current_chat, show);
+	gossip_group_chat_set_show_contacts (GOSSIP_GROUP_CHAT (priv->current_chat), show);
 }
 
 static void
@@ -1048,7 +1029,6 @@ static void
 chat_window_room_set_topic_activate_cb (GtkWidget        *menuitem,
 					GossipChatWindow *window)
 {
-/*FIXME
 	GossipChatWindowPriv *priv;
 	
 	priv = GET_PRIV (window);
@@ -1058,7 +1038,7 @@ chat_window_room_set_topic_activate_cb (GtkWidget        *menuitem,
 
 		group_chat = GOSSIP_GROUP_CHAT (priv->current_chat);
 		gossip_group_chat_set_topic (group_chat);
-	}*/
+	}
 }
 
 static void
@@ -1069,7 +1049,7 @@ chat_window_room_join_new_activate_cb (GtkWidget        *menuitem,
 
 	priv = GET_PRIV (window);
 
-	// FIXME: gossip_new_chatroom_dialog_show (GTK_WINDOW (priv->dialog));
+	gossip_new_chatroom_dialog_show (GTK_WINDOW (priv->dialog));
 }
 
 static void
@@ -1361,6 +1341,7 @@ chat_window_composing_cb (GossipChat       *chat,
 static void
 chat_window_new_message_cb (GossipChat       *chat,
 			    GossipMessage    *message,
+			    gboolean          is_backlog,
 			    GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
@@ -1380,7 +1361,8 @@ chat_window_new_message_cb (GossipChat       *chat,
 
 	needs_urgency = FALSE;
 	if (gossip_chat_is_group_chat (chat)) {		
-		if (gossip_chat_should_highlight_nick (message)) {
+		if (!is_backlog && 
+		    gossip_chat_should_highlight_nick (message)) {
 			gossip_debug (DEBUG_DOMAIN, "Highlight this nick");
 			needs_urgency = TRUE;
 		}
@@ -1392,7 +1374,8 @@ chat_window_new_message_cb (GossipChat       *chat,
 		chat_window_set_urgency_hint (window, TRUE);
 	}
 
-	if (!g_list_find (priv->chats_new_msg, chat)) {
+	if (!is_backlog && 
+	    !g_list_find (priv->chats_new_msg, chat)) {
 		priv->chats_new_msg = g_list_prepend (priv->chats_new_msg, chat);
 		chat_window_update_status (window, chat);
 	}
@@ -1498,16 +1481,16 @@ chat_window_page_added_cb (GtkNotebook      *notebook,
 	gossip_chat_set_window (chat, window);
 
 	/* Connect chat signals for this window */
-	g_signal_connect (chat, "status_changed",
+	g_signal_connect (chat, "status-changed",
 			  G_CALLBACK (chat_window_status_changed_cb),
 			  window);
-	g_signal_connect (chat, "name_changed",
+	g_signal_connect (chat, "name-changed",
 			  G_CALLBACK (chat_window_name_changed_cb),
 			  window);
 	g_signal_connect (chat, "composing",
 			  G_CALLBACK (chat_window_composing_cb),
 			  window);
-	g_signal_connect (chat, "new_message",
+	g_signal_connect (chat, "new-message",
 			  G_CALLBACK (chat_window_new_message_cb),
 			  window);
 
@@ -1606,14 +1589,15 @@ chat_window_drag_data_received (GtkWidget        *widget,
 				guint             time,
 				GossipChatWindow *window)
 {
+	/* FIXME: DnD of contact do not seems to work... */
 	if (info == DND_DRAG_TYPE_CONTACT_ID) {
-#if 0
-FIXME:
-		GossipChatManager *manager;
-		GossipContact     *contact;
-		GossipChat        *chat;
-		GossipChatWindow  *old_window;
-		const gchar       *id = NULL;
+		EmpathyContactManager *manager;
+		GossipContact         *contact;
+		GossipChat            *chat;
+		GossipChatWindow      *old_window;
+		McAccount             *account;
+		const gchar           *id = NULL;
+		gchar                 *chat_id;
 
 		if (selection) {
 			id = (const gchar*) selection->data;
@@ -1621,15 +1605,20 @@ FIXME:
 
 		gossip_debug (DEBUG_DOMAIN, "DND contact from roster with id:'%s'", id);
 		
-		contact = gossip_session_find_contact (gossip_app_get_session (), id);
+		manager = empathy_contact_manager_new ();
+		contact = empathy_contact_list_find (EMPATHY_CONTACT_LIST (manager), id);
+		g_object_unref (manager);
+
 		if (!contact) {
 			gossip_debug (DEBUG_DOMAIN, "DND contact from roster not found");
 			return;
 		}
 		
-		manager = gossip_app_get_chat_manager ();
-		chat = GOSSIP_CHAT (gossip_chat_manager_get_chat (manager, contact));
+		account = gossip_contact_get_account (contact);
+		chat_id = empathy_tp_chat_build_id (account, id);
+		chat = gossip_chat_window_find_chat_by_id (chat_id);
 		old_window = gossip_chat_get_window (chat);
+		g_free (chat_id);
 		
 		if (old_window) {
 			if (old_window == window) {
@@ -1643,7 +1632,7 @@ FIXME:
 		}
 		
 		/* Added to take care of any outstanding chat events */
-		gossip_chat_manager_show_chat (manager, contact);
+		gossip_chat_present (chat);
 
 		/* We should return TRUE to remove the data when doing
 		 * GDK_ACTION_MOVE, but we don't here otherwise it has
@@ -1651,7 +1640,6 @@ FIXME:
 		 * anyway with add_chat() and remove_chat().
 		 */
 		gtk_drag_finish (context, TRUE, FALSE, time);
-#endif
 	}
 	else if (info == DND_DRAG_TYPE_TAB) {
 		GossipChat        *chat = NULL;
@@ -1892,3 +1880,31 @@ gossip_chat_window_has_focus (GossipChatWindow *window)
 
 	return has_focus;
 }
+
+GossipChat *
+gossip_chat_window_find_chat_by_id (const gchar *id)
+{
+	GList *l;
+
+	for (l = chat_windows; l; l = l->next) {
+		GossipChatWindowPriv *priv;
+		GossipChatWindow     *window;
+		GList                *ll;
+
+		window = l->data;
+		priv = GET_PRIV (window);
+
+		for (ll = priv->chats; ll; ll = ll->next) {
+			GossipChat *chat;
+
+			chat = ll->data;
+
+			if (strcmp (id, gossip_chat_get_id (chat)) == 0) {
+				return chat;
+			}
+		}
+	}
+
+	return NULL;
+}
+
