@@ -1072,11 +1072,18 @@ chat_state_changed_cb (EmpathyTpChat             *tp_chat,
 		       GossipChat                *chat)
 {
 	GossipChatPriv *priv;
+	GossipContact  *own_contact;
 	GList          *l;
 	gboolean        was_composing;
 
 	priv = GET_PRIV (chat);
 
+	own_contact = gossip_contact_get_user (contact);
+	if (gossip_contact_equal (own_contact, contact)) {
+		/* We don't care about our own chat state */
+		return;
+	}
+
 	was_composing = (priv->compositors != NULL);
 
 	/* Find the contact in the list. After that l is the list elem or NULL */
